/* $Id: linuxrc.c,v 1.5 2003/12/15 00:37:34 xi Exp $ */

/******************************************************************************
*    nbd_root - Root on network block device.
*    Copyright (C) 2003  Christian Laursen <xi@borderworlds.dk>
*
*    This program is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    This program is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with this program; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
******************************************************************************/


#include <sys/mount.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <linux/unistd.h>
#include <nbdroot_version.h>

int main () {
  struct stat statbuf;
  char *root, *host, *port;
  int pid, i, j;

  fprintf(stderr, "nbd_root version %s\n", NBDROOT_VERSION);
  pid = getpid();
  if (pid != 1) {
    fprintf(stderr, "My pid is %d but should be 1, please make sure that linuxrc gets executed as init.\n", pid);
    return 1;
  }

  if ((root = getenv("nbdroot")) != NULL) {
    fprintf(stderr, "root: %s\n", root);
    j = -1;
    for (i = 0; i < strlen(root); ++i) {
      if (root[i] == ':') {
	j = i;
      }
    }
    if (j == -1) {
      fprintf(stderr, "\"nbdroot\" must be on the form <host>:<port>.\n");
      return 1;
    }
    host = (char *)malloc(strlen(root) + 1);
    strncpy(host, root, j);
    host[j] = 0;
    port = root + j + 1;
  } else {
    fprintf(stderr, "No root specified via the \"nbdroot\" parameter.\n");
    return 1;
  }

  if ((pid = fork())) {
    waitpid(pid,NULL,0);
  } else {
    execl("/bin/nbd-client", "nbd-client", host, port, "/dev/nb0", NULL);
    perror("exec");
  }

  if (mount("/dev/nb0", "/mnt", "ext2", MS_RDONLY, "")) {
    perror("mount");
  }
  if (stat("/mnt/initrd", &statbuf)) {
    fprintf(stderr, "The directory /initrd does not exist on the root filesystem.\nPlease create it and try again.\n");
    return 1;
  }
  if (pivot_root("/mnt","/mnt/initrd")) {
    perror("pivot_root");
  }
  chdir("/");
  execl("/sbin/init", "init", NULL);
  fprintf(stderr, "Failed to start init :(\n");

  return 1;
}
